package de.mattv.fileserver.data;

import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.tasks.UnsupportedFormatException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

@Slf4j
public class Node {
    private static final long MAX_IMAGE_SIZE = 1024*1024*50;
    private static final int PREVIEW_SIZE = 480;

    public final long id;
    public final @NonNull String name;
    public boolean isFile = false;
    public boolean hasPreview = false;
    public long size = 0;
    public Node parent = null;
    public final ArrayList<Node> children = new ArrayList<>();

    public final File file;
    public final File previewFile;

    public Node(long id, @NonNull String name, @NonNull User user) {
        this.id = id;
        this.name = name;
        file = new File(user.userDir, String.valueOf(id));
        previewFile = new File(user.userDir, id + ".png");
    }

    public void removePreview() {
        if (previewFile.exists())
            if (!previewFile.delete())
                log.warn("Failed to delete preview file {}", previewFile);
        hasPreview = false;
    }

    public void createThumbnail() throws IOException {
        if (size > MAX_IMAGE_SIZE)
            return;

        try {
            Thumbnails.of(file)
                    .size(PREVIEW_SIZE, PREVIEW_SIZE)
                    .allowOverwrite(true)
                    .outputFormat("png")
                    .toFile(previewFile);
        } catch (UnsupportedFormatException ignored) {}

        hasPreview = true;
    }
}
