package de.mattv.fileserver.data;

import de.mattv.fileserver.Utils;
import lombok.Getter;
import de.mattv.fileserver.util.NonNull;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

import java.time.Duration;
import java.time.Instant;
import java.util.Collection;
import java.util.List;

public class Token implements Authentication {
    private static final Duration LIFETIME = Duration.ofMinutes(60);
    private static Instant nowPlusLifetime() { return Instant.now().plus(LIFETIME); }

    private @NonNull Instant expiresAt = nowPlusLifetime();
    @Getter private final @NonNull String token;
    @Getter private @NonNull User user;
    @Getter private User sudoRealUser = null;

    public void refresh() { expiresAt = nowPlusLifetime(); }
    public boolean expired() { return Utils.instantExpired(expiresAt); }
    public boolean inSudo() { return sudoRealUser != null; }
    public boolean isAdmin() { return (sudoRealUser != null) || user.admin; }
    public User getRealUser() { return sudoRealUser != null ? sudoRealUser : user; }

    public void unSudo() {
        if (sudoRealUser != null)
            user = sudoRealUser;
        sudoRealUser = null;
    }

    public void sudo(@NonNull User newUser) {
        if (this.sudoRealUser == null) sudoRealUser = user;
        user = newUser;
    }

    public Token(@NonNull String token, @NonNull User user) {
        this.token = token;
        this.user = user;
    }


    @Override public Object getDetails() { return null; }
    @Override public String getName() { return String.valueOf(user.id); }
    @Override public Object getPrincipal() { return user; }
    @Override public Object getCredentials() { return this; }
    @Override public boolean isAuthenticated() { return true; }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return isAdmin() ? List.of(new SimpleGrantedAuthority("ROLE_ADMIN")) : List.of();
    }

    @Override
    public void setAuthenticated(boolean isAuthenticated) throws IllegalArgumentException {
        throw new IllegalArgumentException("Cannot set authenticated");
    }

    @Override
    public String toString() {
        return "Token{" +
                "user=" + user +
                ", real_user=" + sudoRealUser +
                '}';
    }
}
