package de.mattv.fileserver.data.converter;

import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class NodeConverter {
    private static final String NODE_NAME = "node";
    private static final String ATTR_ID_NAME = "id";
    private static final String ATTR_NAME_NAME = "name";
    private static final String ATTR_FLAGS_NAME = "flags";
    private static final String ATTR_SIZE_NAME = "size";

    private static class Flags {
        public static final long FILE = 1;
        public static final long PREVIEW = 2;
    }

    public static void toXml(@NonNull Node node, @NonNull HierarchicalStreamWriter writer) {
        writer.startNode(NODE_NAME);
        long flags = 0;
        if (node.isFile) flags |= Flags.FILE;
        if (node.hasPreview) flags |= Flags.PREVIEW;

        writer.addAttribute(ATTR_ID_NAME, String.valueOf(node.id));
        writer.addAttribute(ATTR_NAME_NAME, node.name);
        writer.addAttribute(ATTR_FLAGS_NAME, String.valueOf(flags));

        if (node.isFile) writer.addAttribute(ATTR_SIZE_NAME, String.valueOf(node.size));

        node.children.forEach(child -> toXml(child, writer));

        writer.endNode();
    }

    public static Node fromXml(@NonNull HierarchicalStreamReader reader, @NonNull User user) {
        reader.moveDown();
        if (!reader.getNodeName().equals(NODE_NAME)) {
            log.error("Trying to parse node from xml type {}", reader.getNodeName());
            ConverterUtils.dumpConversionStackFrames();
        }

        ConverterUtils.push("Node");

        Node node = new Node(
                ConverterUtils.getLong(reader, ATTR_ID_NAME),
                ConverterUtils.getString(reader, ATTR_NAME_NAME),
                user
        );

        ConverterUtils.pop();
        ConverterUtils.push("Node " + node.id);

        long flags = ConverterUtils.getLong(reader, ATTR_FLAGS_NAME);
        node.isFile = (flags & Flags.FILE) != 0;
        if (node.isFile) {
            node.hasPreview = (flags & Flags.PREVIEW) != 0;
            node.size = ConverterUtils.getLong(reader, ATTR_SIZE_NAME);
        }

        while (reader.hasMoreChildren()) {
            Node child = fromXml(reader, user);
            child.parent = node;
            node.children.add(child);
        }

        user.nodes.put(node.id, node);
        ConverterUtils.pop();
        reader.moveUp();

        return node;
    }
}
